//
//  TouchView.h
//  ReflexesTest
//
//  Created by 平井 祐樹 on 12/07/15.
//  Copyright (c) 2012年 __MyCompanyName__. All rights reserved.
//

#import <UIKit/UIKit.h>

/*!
 @protocol TouchViewDelegate
 @abstract ゲーム開始／終了をViewControllerにデリゲートするためのプロトコル
 */
@protocol TouchViewDelegate

/*!
 @method
 @abstract ゲーム開始時に実行されるメソッド
 */
- (void)start;

/*!
 @method
 @abstract ゲーム終了時に実行されるメソッド
 */
- (void)finish;

@end

/*!
 @class TouchView
 @superclass UIView
 @abstract 矩形の表示やタッチイベントの管理を行うUIViewを継承したクラス。
 @discussion 表示領域に3x3のマス目を設定する。マス目はNSDictionaryを用いて表現する。
             例:(x, y) = (0, 0)
             -> [NSDictionary dictionaryWithObjectsAndKeys:
                 [NSNumber numberWithInt:0], @"x",
                 [NSNumber numberWithInt:0], @"y", 
                 nil];
 */
@interface TouchView : UIView
{
    /*!
     @var _points
     @abstract 矩形を配置した座標を保持
     */
    NSMutableSet *_points;
}

/*!
 @property delegate
 @abstract TouchViewDelegateプロトコルを実装したインスタンス
 */
@property (assign, nonatomic) IBOutlet id<TouchViewDelegate> delegate;

/*!
 @method
 @abstract ゲームの開始する前の準備を行うメソッド
 @discussion 引数countで指定された個数分矩形を表示する。
 */
- (void)prepare:(NSInteger)count;

@end

/*!
 @category TouchView(PrivateMethods)
 @abstract TouchViewクラスのプライベートメソッド
 */
@interface TouchView(PrivateMethods)

/*!
 @method 
 @abstract ゲームをクリアしているかどうか判定するメソッド
 @discussion 表示中の矩形をすべてタッチしているか判定する。
             touchesBegan:withEvent:メソッド、touchesMoved:withEvent:メソッドから実行される。
 @param allTouches NSSet UITouchのセット
 */
- (void)judgement:allTouches;

/*!
 @method
 @abstract x,yをセットしたNSDictionayインスタンスを生成する。
 @param x NSInteger x座標
 @param y NSInteger y座標
 @return NSDictionay x,yをセットしたNSDictionayインスタンス
 */
- (NSDictionary *)makePointWithX:(NSInteger)x 
                            andY:(NSInteger)y;

@end

