//
//  TouchView.m
//  ReflexesTest
//
//  Created by 平井 祐樹 on 12/07/15.
//  Copyright (c) 2012年 __MyCompanyName__. All rights reserved.
//

#import "TouchView.h"

// 矩形の幅
#define RECT_WIDTH 100.0f
// 矩形の高さ
#define RECT_HEIHT 100.0f

// ゲームに使用するマスの行数
#define ROW_COUNT 3
// ゲームに使用するマスの列数
#define COL_COUNT 3

@implementation TouchView

@synthesize delegate = _delegate;

#pragma mark - UIViewのメソッド

// 画面がタッチされた瞬間に実行されるメソッド
- (void)touchesBegan:(NSSet *)touches withEvent:(UIEvent *)event
{
    // 判定メソッドを実行する
    [self judgement:[event allTouches]];
}

// 画面にタッチしている指を動かしているときに実行されるメソッド
- (void)touchesMoved:(NSSet *)touches withEvent:(UIEvent *)event
{
    // 指を動かしている間も判定メソッドを実行する
    [self judgement:[event allTouches]];
}

// 画面から指が離れた瞬間に実行されるメソッド
- (void)touchesEnded:(NSSet *)touches withEvent:(UIEvent *)event
{
    // 何もしない
}

// タッチイベントがキャンセルされたときに実行されるメソッド
- (void)touchesCancelled:(NSSet *)touches withEvent:(UIEvent *)event
{
    // 何もしない
}

#pragma mark - 独自のメソッド

- (void)prepare:(NSInteger)count
{    
    // _pointを初期化
    _points  = [[NSMutableSet alloc] init];
    
    // 引数countで指定された数だけ矩形を生成する
    while ([_points count] < count) 
    {
        // 矩形の表示場所をランダムに決定する
        NSDictionary *point = [self makePointWithX:(rand() % ROW_COUNT)
                                              andY:(rand() % COL_COUNT)];
        
        // pointで指定した場所に矩形が配置されていなければ、_pointsにセット
        if ([_points containsObject:point] == NO)
        {
            [_points addObject:point];
        }
    }
    
    // 矩形を描画する
    for (NSDictionary *point in _points)
    { 
        // x・yの値を取り出す
        NSInteger x = [[point objectForKey:@"x"] intValue];
        NSInteger y = [[point objectForKey:@"y"] intValue];
        
        // 矩形用のUIViewを生成する
        UIView *view = [[UIView alloc] init];
        view.backgroundColor = [UIColor redColor];
        view.frame = CGRectMake(x * RECT_WIDTH, y * RECT_HEIHT, RECT_WIDTH, RECT_HEIHT);
        
        // 矩形をサブビューとして追加する
        [self addSubview:view];
    }
    
    // ViewControllerにゲーム開始処理をデリゲートする
    [self.delegate start];
}

- (void)judgement:(NSSet *)allTouches
{
    // _pointsに何もセットされていなければ、まだゲームを開始していないとみなし、何もしない
    if (_points == nil)
    {
        return;
    }
    
    // 配置した矩形の座標コレクションを複製
    NSMutableSet *answers = [NSMutableSet setWithSet:_points];
    
    // タッチ中に指の数だけループ
    for (UITouch *touch in allTouches)
    {
        // タッチイベントの座標を取得する
        CGPoint touchPoint = [touch locationInView:self];
        NSInteger x = floor(touchPoint.x / RECT_WIDTH);
        NSInteger y = floor(touchPoint.y / RECT_HEIHT);
        
        // ゲームで使用する値に変換
        NSDictionary *point = [self makePointWithX:x andY:y];
        
        // タッチした座標が、ゲーム開始時に配置した矩形の座標コレクションに含まれていたら、
        // その座標を複製したコレクションから削除する
        if ([answers containsObject:point] == YES)
        {
            [answers removeObject:point];
        }
    }
    
    // 複製したコレクションが0個なら、すべてタッチできた（ゲームクリア）と見なす
    if ([answers count] == 0)
    {
        _points  = nil;
        
        // 矩形をすべて削除する
        for (UIView *view in [self subviews])
        {
            [view removeFromSuperview];
        }

        // ViewControllerにゲーム終了処理をデリゲートする
        [self.delegate finish];
    }
}

- (NSDictionary *)makePointWithX:(NSInteger)x 
                            andY:(NSInteger)y
{
    // 引数x,yをNSNumberに変換
    NSNumber *xObj = [NSNumber numberWithInt:x];
    NSNumber *yObj = [NSNumber numberWithInt:y];    

    NSDictionary *point = [NSDictionary dictionaryWithObjectsAndKeys:
                           xObj, @"x",
                           yObj, @"y", 
                           nil];
    return point;
}

@end
