//
//  ViewController.h
//  ReflexesTester
//
//  Created by 平井 祐樹 on 12/07/16.
//  Copyright (c) 2012年 __MyCompanyName__. All rights reserved.
//

#import <UIKit/UIKit.h>

#import "TouchView.h"

/*!
 @class ViewController
 @superclass UIViewController
 @abstract ビューコントローラ
 */ 
@interface ViewController : UIViewController<TouchViewDelegate>
{
@private
    /*!
     @var _startDate
     @abstract 反射神経テストの開始時刻を保持するメンバ変数
     */
    NSDate *_startDate;
}

/*!
 @property touchView
 @abstract タッチビュー
 */
@property (weak, nonatomic) IBOutlet TouchView *touchView;

/*!
 @property statusLabel
 @abstract ゲームの開始や終了などの状態を表示するためのラベル
 */
@property (weak, nonatomic) IBOutlet UILabel *statusLabel;

/*!
 @property startButton
 @abstract ゲーム開始ボタン
 */
@property (weak, nonatomic) IBOutlet UIButton *startButton;

/*!
 @method 
 @abstract ゲーム開始ボタンがタップされた時に実行されるメソッド
 @discussion NSTimerクラスのscheduledTimerWithTimeInterval:target:selector:userInfo:repeats:メソッドを
             使用して、1〜10秒後にprepareメソッドを呼び出す。
 */
- (IBAction)startButtonPressed:(id)sender;

@end

/*!
 @category ViewController(PrivateMethods)
 @abstract ViewControllerクラスのプライベートメソッド
 */
@interface ViewController(PrivateMethods)

/*!
 @method
 @abstract ゲームの準備を行うメソッド
 @discussion 表示する矩形の数を1から5の中からランダムで決定しTouchViewに通知する。
 */
- (void)prepare;

/*!
 @method
 @abstract ゲーム開始時に実行されるメソッド
 @discussion ゲームの開始時刻を_startDateにセットする。
             TouchViewDelegateプロトコルのstartメソッドで、ゲームの準備が完了した時に実行される。
 */
- (void)start;

/*!
 @method
 @abstract ゲーム終了時に実行されるメソッド
 @discussion ゲーム開始時刻からの経過時間をstatusLabelにセットする。
             TouchViewDelegateプロトコルのfinishメソッドで、ゲームをクリアした時に実行される。
 */
- (void)finish;

@end